#!/usr/bin/env node

// src/index.ts
import { McpServer as McpServer6 } from "@modelcontextprotocol/sdk/server/mcp.js";
import { StdioServerTransport } from "@modelcontextprotocol/sdk/server/stdio.js";

// src/resources/decks.ts
import { ResourceTemplate } from "@modelcontextprotocol/sdk/server/mcp.js";

// src/utils/ankiClient.ts
import { YankiConnect } from "yanki-connect";
var ankiClient = new YankiConnect({
  autoLaunch: true
});

// src/resources/decks.ts
function registerDeckResources(server2) {
  server2.resource("deck_names", "anki:///decks/names", async (uri) => {
    try {
      const deckNames = await ankiClient.deck.deckNames();
      return {
        contents: [
          {
            uri: uri.href,
            mimeType: "application/json",
            text: JSON.stringify(deckNames, null, 2)
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get deck names: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.resource("deck_names_and_ids", "anki:///decks/names-and-ids", async (uri) => {
    try {
      const deckNamesAndIds = await ankiClient.deck.deckNamesAndIds();
      return {
        contents: [
          {
            uri: uri.href,
            mimeType: "application/json",
            text: JSON.stringify(deckNamesAndIds, null, 2)
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get deck names and IDs: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.resource(
    "deck_config",
    new ResourceTemplate("anki:///decks/{deckName}/config", { list: void 0 }),
    async (uri, { deckName }) => {
      const deckNameString = Array.isArray(deckName) ? deckName[0] : deckName;
      if (!deckNameString) {
        throw new Error("Deck name is required");
      }
      try {
        const config = await ankiClient.deck.getDeckConfig({ deck: deckNameString });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(config, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get deck config for "${deckNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "deck_stats",
    new ResourceTemplate("anki:///decks/{deckNames}/stats", { list: void 0 }),
    async (uri, { deckNames }) => {
      const deckNamesString = Array.isArray(deckNames) ? deckNames[0] : deckNames;
      if (!deckNamesString) {
        throw new Error("Deck names are required");
      }
      const deckNamesArray = deckNamesString.split(",").map((name) => name.trim());
      if (deckNamesArray.length === 0) {
        throw new Error("At least one deck name is required");
      }
      try {
        const stats = await ankiClient.deck.getDeckStats({ decks: deckNamesArray });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(stats, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get deck stats for "${deckNamesArray.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "decks_by_cards",
    new ResourceTemplate("anki:///decks/by-cards/{cardIds}", { list: void 0 }),
    async (uri, { cardIds }) => {
      const cardIdsString = Array.isArray(cardIds) ? cardIds[0] : cardIds;
      if (!cardIdsString) {
        throw new Error("Card IDs are required");
      }
      const cardIdArray = cardIdsString.split(",").map((id) => {
        const num = parseInt(id.trim(), 10);
        if (isNaN(num)) {
          throw new Error(`Invalid card ID: ${id}`);
        }
        return num;
      });
      if (cardIdArray.length === 0) {
        throw new Error("At least one card ID is required");
      }
      try {
        const decks = await ankiClient.deck.getDecks({ cards: cardIdArray });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(decks, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get decks for cards "${cardIdArray.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/resources/cards.ts
import { ResourceTemplate as ResourceTemplate2 } from "@modelcontextprotocol/sdk/server/mcp.js";
function registerCardResources(server2) {
  server2.resource(
    "find_cards",
    new ResourceTemplate2("anki:///cards/search/{query}", { list: void 0 }),
    async (uri) => {
      try {
        const query = decodeURIComponent(uri.pathname.split("/").pop() || "");
        if (!query) {
          throw new Error("Query parameter is required");
        }
        const cardIds = await ankiClient.card.findCards({ query });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(cardIds, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to find cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_info",
    new ResourceTemplate2("anki:///cards/{cardIds}/info", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const cardsInfo = await ankiClient.card.cardsInfo({ cards: cardIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(cardsInfo, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get cards info: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_due_status",
    new ResourceTemplate2("anki:///cards/{cardIds}/due", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const areDue = await ankiClient.card.areDue({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          isDue: areDue[index]
        }));
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(result, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if cards are due: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_suspend_status",
    new ResourceTemplate2("anki:///cards/{cardIds}/suspended", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const areSuspended = await ankiClient.card.areSuspended({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          isSuspended: areSuspended[index]
        }));
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(result, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if cards are suspended: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_mod_time",
    new ResourceTemplate2("anki:///cards/{cardIds}/mod-time", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const modTimes = await ankiClient.card.cardsModTime({ cards: cardIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(modTimes, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get cards modification time: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_to_notes",
    new ResourceTemplate2("anki:///cards/{cardIds}/notes", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const noteIds = await ankiClient.card.cardsToNotes({ cards: cardIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(noteIds, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get note IDs from cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_ease_factors",
    new ResourceTemplate2("anki:///cards/{cardIds}/ease-factors", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const easeFactors = await ankiClient.card.getEaseFactors({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          easeFactor: easeFactors[index]
        }));
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(result, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get ease factors: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_intervals",
    new ResourceTemplate2("anki:///cards/{cardIds}/intervals", { list: void 0 }),
    async (uri) => {
      try {
        const cardIdsParam = uri.pathname.split("/").slice(-2)[0];
        const cardIds = cardIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (cardIds.some((id) => isNaN(id))) {
          throw new Error("Invalid card IDs provided");
        }
        const url = new URL(uri.href);
        const complete = url.searchParams.get("complete") === "true";
        const intervals = await ankiClient.card.getIntervals({ cards: cardIds, complete });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          intervals: intervals[index]
        }));
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(result, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get intervals: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/resources/notes.ts
import { ResourceTemplate as ResourceTemplate3 } from "@modelcontextprotocol/sdk/server/mcp.js";
function registerNoteResources(server2) {
  server2.resource("note_tags", "anki:///notes/tags", async (uri) => {
    try {
      const tags = await ankiClient.note.getTags();
      return {
        contents: [
          {
            uri: uri.href,
            mimeType: "application/json",
            text: JSON.stringify(
              {
                tags: tags.sort(),
                count: tags.length,
                description: "All available tags in the Anki collection"
              },
              null,
              2
            )
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get tags: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.resource(
    "notes_info",
    new ResourceTemplate3("anki:///notes/{noteIds}/info", { list: void 0 }),
    async (uri) => {
      try {
        const noteIdsParam = uri.pathname.split("/").slice(-2)[0];
        const noteIds = noteIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (noteIds.some((id) => isNaN(id))) {
          throw new Error("Invalid note IDs provided");
        }
        const notesInfo = await ankiClient.note.notesInfo({ notes: noteIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(
                {
                  notes: notesInfo,
                  count: notesInfo.length,
                  description: "Detailed information about the requested notes"
                },
                null,
                2
              )
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get notes info: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "notes_search",
    new ResourceTemplate3("anki:///notes/search/{query}", { list: void 0 }),
    async (uri) => {
      try {
        const query = decodeURIComponent(uri.pathname.split("/").pop() || "");
        if (!query) {
          throw new Error("Search query is required");
        }
        const noteIds = await ankiClient.note.findNotes({ query });
        const limitedIds = noteIds.slice(0, 50);
        const notesInfo = limitedIds.length > 0 ? await ankiClient.note.notesInfo({ notes: limitedIds }) : [];
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(
                {
                  query,
                  totalFound: noteIds.length,
                  displayedCount: limitedIds.length,
                  noteIds,
                  notes: notesInfo,
                  description: `Search results for query: "${query}". Showing detailed info for first 50 notes.`
                },
                null,
                2
              )
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to search notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "notes_by_tag",
    new ResourceTemplate3("anki:///notes/tag/{tag}", { list: void 0 }),
    async (uri) => {
      try {
        const tag = decodeURIComponent(uri.pathname.split("/").pop() || "");
        if (!tag) {
          throw new Error("Tag is required");
        }
        const query = `tag:${tag}`;
        const noteIds = await ankiClient.note.findNotes({ query });
        const limitedIds = noteIds.slice(0, 50);
        const notesInfo = limitedIds.length > 0 ? await ankiClient.note.notesInfo({ notes: limitedIds }) : [];
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(
                {
                  tag,
                  totalFound: noteIds.length,
                  displayedCount: limitedIds.length,
                  noteIds,
                  notes: notesInfo,
                  description: `Notes tagged with "${tag}". Showing detailed info for first 50 notes.`
                },
                null,
                2
              )
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get notes by tag: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "notes_recent",
    new ResourceTemplate3("anki:///notes/recent/{days}", { list: void 0 }),
    async (uri) => {
      try {
        const daysParam = uri.pathname.split("/").pop() || "7";
        const days = parseInt(daysParam, 10);
        if (isNaN(days) || days <= 0) {
          throw new Error("Days must be a positive number");
        }
        const query = `edited:${days}`;
        const noteIds = await ankiClient.note.findNotes({ query });
        const limitedIds = noteIds.slice(0, 50);
        const notesInfo = limitedIds.length > 0 ? await ankiClient.note.notesInfo({ notes: limitedIds }) : [];
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(
                {
                  days,
                  totalFound: noteIds.length,
                  displayedCount: limitedIds.length,
                  noteIds,
                  notes: notesInfo,
                  description: `Notes modified in the last ${days} days. Showing detailed info for first 50 notes.`
                },
                null,
                2
              )
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get recent notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "notes_mod_times",
    new ResourceTemplate3("anki:///notes/{noteIds}/mod-times", { list: void 0 }),
    async (uri) => {
      try {
        const noteIdsParam = uri.pathname.split("/").slice(-2)[0];
        const noteIds = noteIdsParam.split(",").map((id) => parseInt(id.trim(), 10));
        if (noteIds.some((id) => isNaN(id))) {
          throw new Error("Invalid note IDs provided");
        }
        const modTimes = await ankiClient.note.notesModTime({ notes: noteIds });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(
                {
                  modificationTimes: modTimes,
                  count: modTimes.length,
                  description: "Modification timestamps for the requested notes"
                },
                null,
                2
              )
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get note modification times: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/resources/model.ts
import { ResourceTemplate as ResourceTemplate4 } from "@modelcontextprotocol/sdk/server/mcp.js";
function registerModelResources(server2) {
  server2.resource("model_names", "anki:///models/names", async (uri) => {
    try {
      const modelNames = await ankiClient.model.modelNames();
      return {
        contents: [
          {
            uri: uri.href,
            mimeType: "application/json",
            text: JSON.stringify(modelNames, null, 2)
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get model names: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.resource("model_names_and_ids", "anki:///models/names-and-ids", async (uri) => {
    try {
      const modelNamesAndIds = await ankiClient.model.modelNamesAndIds();
      return {
        contents: [
          {
            uri: uri.href,
            mimeType: "application/json",
            text: JSON.stringify(modelNamesAndIds, null, 2)
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get model names and IDs: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.resource(
    "models_by_id",
    new ResourceTemplate4("anki:///models/by-id/{modelIds}", { list: void 0 }),
    async (uri, { modelIds }) => {
      const modelIdsString = Array.isArray(modelIds) ? modelIds[0] : modelIds;
      if (!modelIdsString) {
        throw new Error("Model IDs are required");
      }
      const modelIdArray = modelIdsString.split(",").map((id) => {
        const num = parseInt(id.trim(), 10);
        if (isNaN(num)) {
          throw new Error(`Invalid model ID: ${id}`);
        }
        return num;
      });
      if (modelIdArray.length === 0) {
        throw new Error("At least one model ID is required");
      }
      try {
        const models = await ankiClient.model.findModelsById({ modelIds: modelIdArray });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(models, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get models by ID "${modelIdArray.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "models_by_name",
    new ResourceTemplate4("anki:///models/by-name/{modelNames}", { list: void 0 }),
    async (uri, { modelNames }) => {
      const modelNamesString = Array.isArray(modelNames) ? modelNames[0] : modelNames;
      if (!modelNamesString) {
        throw new Error("Model names are required");
      }
      const modelNamesArray = modelNamesString.split(",").map((name) => name.trim());
      if (modelNamesArray.length === 0) {
        throw new Error("At least one model name is required");
      }
      try {
        const models = await ankiClient.model.findModelsByName({ modelNames: modelNamesArray });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(models, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get models by name "${modelNamesArray.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "model_field_names",
    new ResourceTemplate4("anki:///models/{modelName}/fields/names", { list: void 0 }),
    async (uri, { modelName }) => {
      const modelNameString = Array.isArray(modelName) ? modelName[0] : modelName;
      if (!modelNameString) {
        throw new Error("Model name is required");
      }
      try {
        const fieldNames = await ankiClient.model.modelFieldNames({ modelName: modelNameString });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(fieldNames, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get field names for model "${modelNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "model_field_fonts",
    new ResourceTemplate4("anki:///models/{modelName}/fields/fonts", { list: void 0 }),
    async (uri, { modelName }) => {
      const modelNameString = Array.isArray(modelName) ? modelName[0] : modelName;
      if (!modelNameString) {
        throw new Error("Model name is required");
      }
      try {
        const fieldFonts = await ankiClient.model.modelFieldFonts({ modelName: modelNameString });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(fieldFonts, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get field fonts for model "${modelNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "model_fields_on_templates",
    new ResourceTemplate4("anki:///models/{modelName}/fields/templates", { list: void 0 }),
    async (uri, { modelName }) => {
      const modelNameString = Array.isArray(modelName) ? modelName[0] : modelName;
      if (!modelNameString) {
        throw new Error("Model name is required");
      }
      try {
        const fieldsOnTemplates = await ankiClient.model.modelFieldsOnTemplates({
          modelName: modelNameString
        });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(fieldsOnTemplates, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get fields on templates for model "${modelNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "model_styling",
    new ResourceTemplate4("anki:///models/{modelName}/styling", { list: void 0 }),
    async (uri, { modelName }) => {
      const modelNameString = Array.isArray(modelName) ? modelName[0] : modelName;
      if (!modelNameString) {
        throw new Error("Model name is required");
      }
      try {
        const styling = await ankiClient.model.modelStyling({ modelName: modelNameString });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(styling, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get styling for model "${modelNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "model_templates",
    new ResourceTemplate4("anki:///models/{modelName}/templates", { list: void 0 }),
    async (uri, { modelName }) => {
      const modelNameString = Array.isArray(modelName) ? modelName[0] : modelName;
      if (!modelNameString) {
        throw new Error("Model name is required");
      }
      try {
        const templates = await ankiClient.model.modelTemplates({ modelName: modelNameString });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(templates, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get templates for model "${modelNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/resources/statistic.ts
import { ResourceTemplate as ResourceTemplate5 } from "@modelcontextprotocol/sdk/server/mcp.js";
function registerStatisticResources(server2) {
  server2.resource(
    "cards_reviewed_by_day",
    "anki:///statistics/cards-reviewed-by-day",
    async (uri) => {
      try {
        const reviewsByDay = await ankiClient.statistic.getNumCardsReviewedByDay();
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(reviewsByDay, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get cards reviewed by day: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "cards_reviewed_today",
    "anki:///statistics/cards-reviewed-today",
    async (uri) => {
      try {
        const reviewsToday = await ankiClient.statistic.getNumCardsReviewedToday();
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify({ cardsReviewedToday: reviewsToday }, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get cards reviewed today: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "card_reviews",
    new ResourceTemplate5("anki:///statistics/decks/{deckName}/reviews/{startID}", {
      list: void 0
    }),
    async (uri, { deckName, startID }) => {
      const deckNameString = Array.isArray(deckName) ? deckName[0] : deckName;
      const startIDString = Array.isArray(startID) ? startID[0] : startID;
      if (!deckNameString) {
        throw new Error("Deck name is required");
      }
      if (!startIDString) {
        throw new Error("Start ID is required");
      }
      const startIDNumber = parseInt(startIDString, 10);
      if (isNaN(startIDNumber)) {
        throw new Error(`Invalid start ID: ${startIDString}`);
      }
      try {
        const reviews = await ankiClient.statistic.cardReviews({
          deck: deckNameString,
          startID: startIDNumber
        });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(reviews, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get card reviews for deck "${deckNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "latest_review_id",
    new ResourceTemplate5("anki:///statistics/decks/{deckName}/latest-review-id", {
      list: void 0
    }),
    async (uri, { deckName }) => {
      const deckNameString = Array.isArray(deckName) ? deckName[0] : deckName;
      if (!deckNameString) {
        throw new Error("Deck name is required");
      }
      try {
        const reviewID = await ankiClient.statistic.getLatestReviewID({ deck: deckNameString });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(
                {
                  deck: deckNameString,
                  latestReviewID: reviewID,
                  hasReviews: reviewID > 0
                },
                null,
                2
              )
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get latest review ID for deck "${deckNameString}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "reviews_of_cards",
    new ResourceTemplate5("anki:///statistics/cards/{cardIds}/reviews", { list: void 0 }),
    async (uri, { cardIds }) => {
      const cardIdsString = Array.isArray(cardIds) ? cardIds[0] : cardIds;
      if (!cardIdsString) {
        throw new Error("Card IDs are required");
      }
      const cardIdArray = cardIdsString.split(",").map((id) => id.trim());
      if (cardIdArray.length === 0) {
        throw new Error("At least one card ID is required");
      }
      try {
        const reviews = await ankiClient.statistic.getReviewsOfCards({ cards: cardIdArray });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "application/json",
              text: JSON.stringify(reviews, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get reviews for cards "${cardIdArray.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.resource(
    "collection_stats_html",
    new ResourceTemplate5("anki:///statistics/collection/{wholeCollection}", { list: void 0 }),
    async (uri, { wholeCollection }) => {
      const wholeCollectionString = Array.isArray(wholeCollection) ? wholeCollection[0] : wholeCollection;
      if (!wholeCollectionString) {
        throw new Error("wholeCollection parameter is required (true/false)");
      }
      const isWholeCollection = wholeCollectionString.toLowerCase() === "true";
      try {
        const statsHTML = await ankiClient.statistic.getCollectionStatsHTML({
          wholeCollection: isWholeCollection
        });
        return {
          contents: [
            {
              uri: uri.href,
              mimeType: "text/html",
              text: statsHTML
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get collection statistics: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/tools/decks.ts
import { z } from "zod";
function registerDeckTools(server2) {
  server2.tool(
    "create_deck",
    {
      deckName: z.string().describe("Name of the deck to create")
    },
    async ({ deckName }) => {
      try {
        const result = await ankiClient.deck.createDeck({ deck: deckName });
        return {
          content: [
            {
              type: "text",
              text: `Successfully created deck "${deckName}". Result: ${JSON.stringify(result)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to create deck "${deckName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "delete_decks",
    {
      deckNames: z.array(z.string()).describe("Array of deck names to delete"),
      deleteCards: z.boolean().default(true).describe("Whether to delete the cards in the decks as well")
    },
    async ({ deckNames, deleteCards }) => {
      try {
        await ankiClient.deck.deleteDecks({
          decks: deckNames,
          cardsToo: deleteCards
          // Type assertion needed due to yanki-connect's strict typing
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully deleted decks: ${deckNames.join(", ")}${deleteCards ? " (including cards)" : " (cards preserved)"}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to delete decks "${deckNames.join(", ")}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "change_deck",
    {
      cardIds: z.array(z.number()).describe("Array of card IDs to move"),
      targetDeck: z.string().describe("Name of the target deck to move cards to")
    },
    async ({ cardIds, targetDeck }) => {
      try {
        await ankiClient.deck.changeDeck({
          cards: cardIds,
          deck: targetDeck
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully moved ${cardIds.length} cards to deck "${targetDeck}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to move cards to deck "${targetDeck}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "clone_deck_config",
    {
      sourceConfigId: z.number().describe("ID of the deck configuration to clone from"),
      newConfigName: z.string().describe("Name for the new cloned configuration")
    },
    async ({ sourceConfigId, newConfigName }) => {
      try {
        const result = await ankiClient.deck.cloneDeckConfigId({
          cloneFrom: sourceConfigId,
          name: newConfigName
        });
        if (result === false) {
          throw new Error("Failed to clone deck configuration - operation returned false");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully cloned deck configuration. New config ID: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to clone deck configuration: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "remove_deck_config",
    {
      configId: z.number().describe("ID of the deck configuration to remove")
    },
    async ({ configId }) => {
      try {
        const result = await ankiClient.deck.removeDeckConfigId({ configId });
        if (!result) {
          throw new Error("Failed to remove deck configuration - operation returned false");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully removed deck configuration with ID: ${configId}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to remove deck configuration: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "save_deck_config",
    {
      config: z.object({
        id: z.number(),
        name: z.string(),
        autoplay: z.boolean(),
        dyn: z.boolean(),
        maxTaken: z.number(),
        mod: z.number(),
        replayq: z.boolean(),
        timer: z.number(),
        usn: z.number(),
        lapse: z.object({
          delays: z.array(z.number()),
          leechAction: z.number(),
          leechFails: z.number(),
          minInt: z.number(),
          mult: z.number()
        }),
        new: z.object({
          bury: z.boolean(),
          delays: z.array(z.number()),
          initialFactor: z.number(),
          ints: z.array(z.number()),
          order: z.number(),
          perDay: z.number(),
          separate: z.boolean()
        }),
        rev: z.object({
          bury: z.boolean(),
          ease4: z.number(),
          fuzz: z.number(),
          ivlFct: z.number(),
          maxIvl: z.number(),
          minSpace: z.number(),
          perDay: z.number()
        })
      }).describe("Complete deck configuration object to save")
    },
    async ({ config }) => {
      try {
        const result = await ankiClient.deck.saveDeckConfig({ config });
        if (!result) {
          throw new Error("Failed to save deck configuration - operation returned false");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully saved deck configuration "${config.name}" (ID: ${config.id})`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to save deck configuration: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "set_deck_config",
    {
      configId: z.number().describe("ID of the configuration to apply"),
      deckNames: z.array(z.string()).describe("Array of deck names to apply the configuration to")
    },
    async ({ configId, deckNames }) => {
      try {
        const result = await ankiClient.deck.setDeckConfigId({
          configId,
          decks: deckNames
        });
        if (!result) {
          throw new Error("Failed to set deck configuration - operation returned false");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully applied configuration ${configId} to decks: ${deckNames.join(", ")}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set deck configuration: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/tools/cards.ts
import { z as z2 } from "zod";

// src/utils/bulkOperations.ts
async function executeBulkOperation(items, operation, getItemId = (_, index) => index) {
  const results = await Promise.allSettled(items.map((item) => operation(item)));
  const result = {
    totalOperations: items.length,
    successful: 0,
    failed: 0,
    errors: []
  };
  results.forEach((promiseResult, index) => {
    if (promiseResult.status === "fulfilled") {
      result.successful++;
    } else {
      result.failed++;
      result.errors.push({
        id: getItemId(items[index], index),
        error: promiseResult.reason instanceof Error ? promiseResult.reason.message : String(promiseResult.reason)
      });
    }
  });
  return result;
}
function formatBulkOperationResult(result, operationName, showErrors = true) {
  let message = `${operationName} completed:
`;
  message += `- Total operations: ${result.totalOperations}
`;
  message += `- Successful: ${result.successful}
`;
  message += `- Failed: ${result.failed}`;
  if (showErrors && result.errors.length > 0) {
    message += "\n\nErrors:\n";
    result.errors.slice(0, 10).forEach((error) => {
      message += `- ID ${error.id}: ${error.error}
`;
    });
    if (result.errors.length > 10) {
      message += `... and ${result.errors.length - 10} more errors`;
    }
  }
  return message;
}

// src/tools/cards.ts
function registerCardTools(server2) {
  server2.tool(
    "answer_cards",
    {
      answers: z2.array(
        z2.object({
          cardId: z2.number().describe("ID of the card to answer"),
          ease: z2.number().min(1).max(4).describe("Ease rating: 1 (Again), 2 (Hard), 3 (Good), 4 (Easy)")
        })
      ).describe("Array of card answers")
    },
    async ({ answers }) => {
      try {
        const results = await ankiClient.card.answerCards({ answers });
        const successCount = results.filter(Boolean).length;
        const failureCount = results.length - successCount;
        return {
          content: [
            {
              type: "text",
              text: `Answered ${successCount} cards successfully, ${failureCount} failed. Results: ${JSON.stringify(results)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to answer cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "find_cards",
    {
      query: z2.string().describe("Anki search query to find cards")
    },
    async ({ query }) => {
      try {
        const cardIds = await ankiClient.card.findCards({ query });
        return {
          content: [
            {
              type: "text",
              text: `Found ${cardIds.length} cards matching query "${query}": ${JSON.stringify(cardIds)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to find cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_cards_info",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get information for")
    },
    async ({ cardIds }) => {
      try {
        const cardsInfo = await ankiClient.card.cardsInfo({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Card information: ${JSON.stringify(cardsInfo, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get cards info: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "check_cards_due",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to check")
    },
    async ({ cardIds }) => {
      try {
        const areDue = await ankiClient.card.areDue({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          isDue: areDue[index]
        }));
        const dueCount = areDue.filter(Boolean).length;
        return {
          content: [
            {
              type: "text",
              text: `${dueCount} out of ${cardIds.length} cards are due. Details: ${JSON.stringify(result)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if cards are due: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "check_cards_suspended",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to check")
    },
    async ({ cardIds }) => {
      try {
        const areSuspended = await ankiClient.card.areSuspended({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          isSuspended: areSuspended[index]
        }));
        const suspendedCount = areSuspended.filter((status) => status === true).length;
        return {
          content: [
            {
              type: "text",
              text: `${suspendedCount} out of ${cardIds.length} cards are suspended. Details: ${JSON.stringify(result)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if cards are suspended: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "suspend_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to suspend")
    },
    async ({ cardIds }) => {
      try {
        const result = await ankiClient.card.suspend({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully suspended ${cardIds.length} cards. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to suspend cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "unsuspend_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to unsuspend")
    },
    async ({ cardIds }) => {
      try {
        const result = await ankiClient.card.unsuspend({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully unsuspended ${cardIds.length} cards. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to unsuspend cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "check_card_suspended",
    {
      cardId: z2.number().describe("Card ID to check")
    },
    async ({ cardId }) => {
      try {
        const isSuspended = await ankiClient.card.suspended({ card: cardId });
        return {
          content: [
            {
              type: "text",
              text: `Card ${cardId} is ${isSuspended ? "suspended" : "not suspended"}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to check if card is suspended: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "forget_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to forget")
    },
    async ({ cardIds }) => {
      try {
        await ankiClient.card.forgetCards({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully forgot ${cardIds.length} cards, making them new again`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to forget cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "relearn_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to relearn")
    },
    async ({ cardIds }) => {
      try {
        await ankiClient.card.relearnCards({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully set ${cardIds.length} cards to relearn`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to relearn cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "set_cards_due_date",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to set due date for"),
      days: z2.string().describe("Number of days from today (can be negative for past dates)")
    },
    async ({ cardIds, days }) => {
      try {
        const result = await ankiClient.card.setDueDate({ cards: cardIds, days });
        return {
          content: [
            {
              type: "text",
              text: `Successfully set due date for cards. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set due date for cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_cards_ease_factors",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get ease factors for")
    },
    async ({ cardIds }) => {
      try {
        const easeFactors = await ankiClient.card.getEaseFactors({ cards: cardIds });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          easeFactor: easeFactors[index]
        }));
        return {
          content: [
            {
              type: "text",
              text: `Ease factors: ${JSON.stringify(result, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get ease factors: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "set_cards_ease_factors",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to set ease factors for"),
      easeFactors: z2.array(z2.number()).describe("Array of ease factors (must match the length of cardIds)")
    },
    async ({ cardIds, easeFactors }) => {
      try {
        if (cardIds.length !== easeFactors.length) {
          throw new Error("Number of card IDs must match number of ease factors");
        }
        const results = await ankiClient.card.setEaseFactors({
          cards: cardIds,
          easeFactors
        });
        const successCount = results.filter(Boolean).length;
        return {
          content: [
            {
              type: "text",
              text: `Successfully set ease factors for ${successCount} out of ${cardIds.length} cards`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set ease factors: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_cards_intervals",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get intervals for"),
      complete: z2.boolean().optional().describe("If true, returns all intervals; if false, returns only the most recent")
    },
    async ({ cardIds, complete = false }) => {
      try {
        const intervals = await ankiClient.card.getIntervals({ cards: cardIds, complete });
        const result = cardIds.map((cardId, index) => ({
          cardId,
          intervals: intervals[index]
        }));
        return {
          content: [
            {
              type: "text",
              text: `Intervals (${complete ? "complete history" : "most recent"}): ${JSON.stringify(result, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get intervals: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_cards_mod_time",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get modification time for")
    },
    async ({ cardIds }) => {
      try {
        const modTimes = await ankiClient.card.cardsModTime({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Modification times: ${JSON.stringify(modTimes, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get modification times: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "cards_to_notes",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to get note IDs for")
    },
    async ({ cardIds }) => {
      try {
        const noteIds = await ankiClient.card.cardsToNotes({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Note IDs: ${JSON.stringify(noteIds)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get note IDs from cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "set_card_specific_values",
    {
      cardId: z2.number().describe("Card ID to modify"),
      keys: z2.array(
        z2.enum([
          "data",
          "did",
          "due",
          "factor",
          "flags",
          "id",
          "ivl",
          "lapses",
          "left",
          "mod",
          "odid",
          "odue",
          "ord",
          "queue",
          "reps",
          "type",
          "usn"
        ])
      ).describe("Array of card property keys to modify"),
      newValues: z2.array(z2.string()).describe("Array of new values (must match the length of keys)")
    },
    async ({ cardId, keys, newValues }) => {
      try {
        if (keys.length !== newValues.length) {
          throw new Error("Number of keys must match number of new values");
        }
        const results = await ankiClient.card.setSpecificValueOfCard({
          card: cardId,
          keys,
          newValues
        });
        const successCount = results.filter(Boolean).length;
        return {
          content: [
            {
              type: "text",
              text: `Successfully updated ${successCount} out of ${keys.length} values for card ${cardId}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set card values: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "bulk_move_cards_to_deck",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to move"),
      deckName: z2.string().describe("Name of the destination deck")
    },
    async ({ cardIds, deckName }) => {
      try {
        const result = await executeBulkOperation(
          cardIds,
          async (cardId) => {
            await ankiClient.deck.changeDeck({
              cards: [cardId],
              deck: deckName
            });
          },
          (cardId) => `Card-${cardId}`
        );
        const message = formatBulkOperationResult(result, `Bulk move cards to deck "${deckName}"`);
        return {
          content: [
            {
              type: "text",
              text: message
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to bulk move cards to deck: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "bulk_suspend_cards",
    {
      cardIds: z2.array(z2.number()).describe("Array of card IDs to suspend/unsuspend"),
      suspend: z2.boolean().describe("Whether to suspend (true) or unsuspend (false) the cards")
    },
    async ({ cardIds, suspend }) => {
      try {
        const result = await executeBulkOperation(
          cardIds,
          async (cardId) => {
            if (suspend) {
              await ankiClient.card.suspend({ cards: [cardId] });
            } else {
              await ankiClient.card.unsuspend({ cards: [cardId] });
            }
          },
          (cardId) => `Card-${cardId}`
        );
        const action = suspend ? "suspend" : "unsuspend";
        const message = formatBulkOperationResult(result, `Bulk ${action} cards`);
        return {
          content: [
            {
              type: "text",
              text: message
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to bulk ${suspend ? "suspend" : "unsuspend"} cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/tools/notes.ts
import { z as z3 } from "zod";
function registerNoteTools(server2) {
  server2.tool(
    "add_note",
    {
      deckName: z3.string().describe("Name of the deck to add the note to"),
      modelName: z3.string().describe('Name of the note model/type (e.g., "Basic", "Cloze")'),
      fields: z3.record(z3.string()).describe("Object with field names as keys and field content as values"),
      tags: z3.array(z3.string()).optional().describe("Array of tags to add to the note")
    },
    async ({ deckName, modelName, fields, tags }) => {
      try {
        const note = {
          deckName,
          modelName,
          fields,
          tags: tags || []
        };
        const noteId = await ankiClient.note.addNote({ note });
        if (noteId === null) {
          throw new Error("Failed to add note - possibly a duplicate or invalid fields");
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully added note with ID: ${noteId}. Deck: "${deckName}", Model: "${modelName}", Tags: [${tags?.join(", ") || "none"}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to add note: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "add_notes",
    {
      notes: z3.array(
        z3.object({
          deckName: z3.string().describe("Name of the deck to add the note to"),
          modelName: z3.string().describe("Name of the note model/type"),
          fields: z3.record(z3.string()).describe("Object with field names as keys and field content as values"),
          tags: z3.array(z3.string()).optional().describe("Array of tags to add to the note")
        })
      ).describe("Array of notes to add")
    },
    async ({ notes }) => {
      try {
        const formattedNotes = notes.map((note) => ({
          ...note,
          tags: note.tags || []
        }));
        const results = await ankiClient.note.addNotes({ notes: formattedNotes });
        const successCount = results?.filter((result) => result !== null).length || 0;
        const failureCount = (results?.length || 0) - successCount;
        return {
          content: [
            {
              type: "text",
              text: `Batch note addition completed. Successfully added: ${successCount} notes, Failed: ${failureCount} notes. Results: ${JSON.stringify(results)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to add notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "update_note",
    {
      noteId: z3.number().describe("ID of the note to update"),
      fields: z3.record(z3.string()).optional().describe("Object with field names as keys and new field content as values"),
      tags: z3.array(z3.string()).optional().describe("Array of tags to set for the note")
    },
    async ({ noteId, fields, tags }) => {
      try {
        if (!fields && !tags) {
          throw new Error("Either fields or tags must be provided for update");
        }
        const updateData = { id: noteId };
        if (fields) updateData.fields = fields;
        if (tags) updateData.tags = tags;
        await ankiClient.note.updateNote({ note: updateData });
        return {
          content: [
            {
              type: "text",
              text: `Successfully updated note ${noteId}. ${fields ? "Updated fields. " : ""}${tags ? `Updated tags: [${tags.join(", ")}]` : ""}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to update note ${noteId}: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "delete_notes",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to delete")
    },
    async ({ noteIds }) => {
      try {
        await ankiClient.note.deleteNotes({ notes: noteIds });
        return {
          content: [
            {
              type: "text",
              text: `Successfully deleted ${noteIds.length} notes: [${noteIds.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to delete notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "add_tags_to_notes",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to add tags to"),
      tags: z3.string().describe("Space-separated string of tags to add")
    },
    async ({ noteIds, tags }) => {
      try {
        await ankiClient.note.addTags({ notes: noteIds, tags });
        return {
          content: [
            {
              type: "text",
              text: `Successfully added tags "${tags}" to ${noteIds.length} notes: [${noteIds.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to add tags to notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "remove_tags_from_notes",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to remove tags from"),
      tags: z3.string().describe("Space-separated string of tags to remove")
    },
    async ({ noteIds, tags }) => {
      try {
        await ankiClient.note.removeTags({ notes: noteIds, tags });
        return {
          content: [
            {
              type: "text",
              text: `Successfully removed tags "${tags}" from ${noteIds.length} notes: [${noteIds.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to remove tags from notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "replace_tags_in_notes",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to replace tags in"),
      tagToReplace: z3.string().describe("Tag to replace"),
      replaceWithTag: z3.string().describe("Tag to replace with")
    },
    async ({ noteIds, tagToReplace, replaceWithTag }) => {
      try {
        await ankiClient.note.replaceTags({
          notes: noteIds,
          tag_to_replace: tagToReplace,
          replace_with_tag: replaceWithTag
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully replaced tag "${tagToReplace}" with "${replaceWithTag}" in ${noteIds.length} notes: [${noteIds.join(", ")}]`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to replace tags in notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "find_notes",
    {
      query: z3.string().describe("Anki search query to find notes"),
      includeInfo: z3.boolean().default(false).describe("Whether to include detailed note information"),
      limit: z3.number().default(50).describe("Maximum number of notes to return detailed info for")
    },
    async ({ query, includeInfo, limit }) => {
      try {
        const noteIds = await ankiClient.note.findNotes({ query });
        let result = `Found ${noteIds.length} notes matching query: "${query}"`;
        if (includeInfo && noteIds.length > 0) {
          const limitedIds = noteIds.slice(0, limit);
          const notesInfo = await ankiClient.note.notesInfo({ notes: limitedIds });
          result += `

Detailed information for first ${limitedIds.length} notes:
${JSON.stringify(notesInfo, null, 2)}`;
        } else {
          result += `

Note IDs: [${noteIds.join(", ")}]`;
        }
        return {
          content: [
            {
              type: "text",
              text: result
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to find notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "can_add_notes",
    {
      notes: z3.array(
        z3.object({
          deckName: z3.string().describe("Name of the deck to add the note to"),
          modelName: z3.string().describe("Name of the note model/type"),
          fields: z3.record(z3.string()).describe("Object with field names as keys and field content as values"),
          tags: z3.array(z3.string()).optional().describe("Array of tags to add to the note")
        })
      ).describe("Array of notes to validate")
    },
    async ({ notes }) => {
      try {
        const formattedNotes = notes.map((note) => ({
          ...note,
          tags: note.tags || []
        }));
        const canAddResults = await ankiClient.note.canAddNotes({ notes: formattedNotes });
        const validCount = canAddResults.filter((result) => result).length;
        const invalidCount = canAddResults.length - validCount;
        return {
          content: [
            {
              type: "text",
              text: `Note validation completed. Valid: ${validCount}, Invalid: ${invalidCount}. Results: ${JSON.stringify(canAddResults)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to validate notes: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool("clear_unused_tags", {}, async () => {
    try {
      const removedTags = await ankiClient.note.clearUnusedTags();
      return {
        content: [
          {
            type: "text",
            text: `Successfully cleared ${removedTags.length} unused tags: [${removedTags.join(", ")}]`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to clear unused tags: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("remove_empty_notes", {}, async () => {
    try {
      await ankiClient.note.removeEmptyNotes();
      return {
        content: [
          {
            type: "text",
            text: "Successfully removed all empty notes from the collection"
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to remove empty notes: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool(
    "bulk_add_tags",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to add tags to"),
      tags: z3.array(z3.string()).min(1).describe("Array of tags to add")
    },
    async ({ noteIds, tags }) => {
      try {
        const tagsString = tags.join(" ");
        const result = await executeBulkOperation(
          noteIds,
          async (noteId) => {
            await ankiClient.note.addTags({
              notes: [noteId],
              tags: tagsString
            });
          },
          (noteId) => `Note-${noteId}`
        );
        const message = formatBulkOperationResult(result, `Bulk add tags [${tags.join(", ")}]`);
        return {
          content: [
            {
              type: "text",
              text: message
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to bulk add tags: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "bulk_remove_tags",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to remove tags from"),
      tags: z3.array(z3.string()).min(1).describe("Array of tags to remove")
    },
    async ({ noteIds, tags }) => {
      try {
        const tagsString = tags.join(" ");
        const result = await executeBulkOperation(
          noteIds,
          async (noteId) => {
            await ankiClient.note.removeTags({
              notes: [noteId],
              tags: tagsString
            });
          },
          (noteId) => `Note-${noteId}`
        );
        const message = formatBulkOperationResult(result, `Bulk remove tags [${tags.join(", ")}]`);
        return {
          content: [
            {
              type: "text",
              text: message
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to bulk remove tags: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "bulk_update_note_fields",
    {
      noteIds: z3.array(z3.number()).describe("Array of note IDs to update"),
      fields: z3.record(z3.string()).describe("Object with field names as keys and new content as values")
    },
    async ({ noteIds, fields }) => {
      try {
        const result = await executeBulkOperation(
          noteIds,
          async (noteId) => {
            await ankiClient.note.updateNoteFields({
              note: {
                id: noteId,
                fields
              }
            });
          },
          (noteId) => `Note-${noteId}`
        );
        const fieldsList = Object.keys(fields).join(", ");
        const message = formatBulkOperationResult(result, `Bulk update fields [${fieldsList}]`);
        return {
          content: [
            {
              type: "text",
              text: message
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to bulk update note fields: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/tools/graphical.ts
import { z as z4 } from "zod";
function registerGraphicalTools(server2) {
  server2.tool(
    "gui_add_cards",
    {
      deckName: z4.string().describe("Name of the deck to add the note to"),
      modelName: z4.string().describe("Name of the note model/type"),
      fields: z4.record(z4.string()).describe("Object with field names as keys and field content as values"),
      tags: z4.array(z4.string()).optional().describe("Array of tags to add to the note")
    },
    async ({ deckName, modelName, fields, tags }) => {
      try {
        const note = {
          deckName,
          modelName,
          fields,
          tags: tags || []
        };
        const result = await ankiClient.graphical.guiAddCards({ note });
        return {
          content: [
            {
              type: "text",
              text: `Opened Add Cards dialog and added note with ID: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to open Add Cards dialog: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "gui_answer_card",
    {
      ease: z4.number().min(1).max(4).describe("Ease rating: 1 (Again), 2 (Hard), 3 (Good), 4 (Easy)")
    },
    async ({ ease }) => {
      try {
        const result = await ankiClient.graphical.guiAnswerCard({ ease });
        return {
          content: [
            {
              type: "text",
              text: `Successfully answered current card with ease ${ease}. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to answer card: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "gui_browse",
    {
      query: z4.string().describe("Search query to browse cards"),
      reorderCards: z4.object({
        columnId: z4.string().describe("Column to sort by"),
        order: z4.enum(["ascending", "descending"]).describe("Sort order")
      }).optional().describe("Optional reordering configuration")
    },
    async ({ query, reorderCards }) => {
      try {
        const params = { query };
        if (reorderCards) {
          params.reorderCards = reorderCards;
        }
        const cardIds = await ankiClient.graphical.guiBrowse(params);
        return {
          content: [
            {
              type: "text",
              text: `Opened browser with query "${query}". Found ${cardIds.length} cards: ${JSON.stringify(cardIds)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to open browser: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool("gui_check_database", {}, async () => {
    try {
      const result = await ankiClient.graphical.guiCheckDatabase();
      return {
        content: [
          {
            type: "text",
            text: `Database check completed successfully. Result: ${result}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to check database: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("gui_current_card", {}, async () => {
    try {
      const cardInfo = await ankiClient.graphical.guiCurrentCard();
      return {
        content: [
          {
            type: "text",
            text: cardInfo ? `Current card: ${JSON.stringify(cardInfo, null, 2)}` : "No card is currently being reviewed"
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get current card: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("gui_deck_browser", {}, async () => {
    try {
      await ankiClient.graphical.guiDeckBrowser();
      return {
        content: [
          {
            type: "text",
            text: "Successfully opened deck browser"
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to open deck browser: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool(
    "gui_deck_overview",
    {
      deckName: z4.string().describe("Name of the deck to view overview for")
    },
    async ({ deckName }) => {
      try {
        const result = await ankiClient.graphical.guiDeckOverview({ name: deckName });
        return {
          content: [
            {
              type: "text",
              text: `Successfully opened deck overview for "${deckName}". Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to open deck overview: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "gui_deck_review",
    {
      deckName: z4.string().describe("Name of the deck to start reviewing")
    },
    async ({ deckName }) => {
      try {
        const result = await ankiClient.graphical.guiDeckReview({ name: deckName });
        return {
          content: [
            {
              type: "text",
              text: `Successfully started review for deck "${deckName}". Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to start deck review: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "gui_edit_note",
    {
      noteId: z4.number().describe("ID of the note to edit")
    },
    async ({ noteId }) => {
      try {
        await ankiClient.graphical.guiEditNote({ note: noteId });
        return {
          content: [
            {
              type: "text",
              text: `Successfully opened edit dialog for note ${noteId}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to edit note: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool("gui_exit_anki", {}, async () => {
    try {
      await ankiClient.graphical.guiExitAnki();
      return {
        content: [
          {
            type: "text",
            text: "Successfully sent exit command to Anki"
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to exit Anki: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool(
    "gui_import_file",
    {
      filePath: z4.string().describe("Path to the file to import")
    },
    async ({ filePath }) => {
      try {
        await ankiClient.graphical.guiImportFile({ path: filePath });
        return {
          content: [
            {
              type: "text",
              text: `Successfully imported file: ${filePath}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to import file: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "gui_select_card",
    {
      cardId: z4.number().describe("ID of the card to select")
    },
    async ({ cardId }) => {
      try {
        const result = await ankiClient.graphical.guiSelectCard({ card: cardId });
        return {
          content: [
            {
              type: "text",
              text: `Successfully selected card ${cardId}. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to select card: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "gui_select_note",
    {
      noteId: z4.number().describe("ID of the note to select")
    },
    async ({ noteId }) => {
      try {
        const result = await ankiClient.graphical.guiSelectNote({ note: noteId });
        return {
          content: [
            {
              type: "text",
              text: `Successfully selected note ${noteId}. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to select note: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool("gui_selected_notes", {}, async () => {
    try {
      const noteIds = await ankiClient.graphical.guiSelectedNotes();
      return {
        content: [
          {
            type: "text",
            text: `Selected notes: ${JSON.stringify(noteIds)}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get selected notes: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("gui_show_answer", {}, async () => {
    try {
      const result = await ankiClient.graphical.guiShowAnswer();
      return {
        content: [
          {
            type: "text",
            text: `Successfully showed answer. Result: ${result}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to show answer: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("gui_show_question", {}, async () => {
    try {
      const result = await ankiClient.graphical.guiShowQuestion();
      return {
        content: [
          {
            type: "text",
            text: `Successfully showed question. Result: ${result}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to show question: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("gui_start_card_timer", {}, async () => {
    try {
      const result = await ankiClient.graphical.guiStartCardTimer();
      return {
        content: [
          {
            type: "text",
            text: `Successfully started card timer. Result: ${result}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to start card timer: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("gui_undo", {}, async () => {
    try {
      const result = await ankiClient.graphical.guiUndo();
      return {
        content: [
          {
            type: "text",
            text: `Successfully performed undo. Result: ${result}`
          }
        ]
      };
    } catch (error) {
      throw new Error(`Failed to undo: ${error instanceof Error ? error.message : String(error)}`);
    }
  });
}

// src/tools/media.ts
import { z as z5 } from "zod";
function registerMediaTools(server2) {
  server2.tool(
    "delete_media_file",
    {
      filename: z5.string().describe("Name of the media file to delete")
    },
    async ({ filename }) => {
      try {
        await ankiClient.media.deleteMediaFile({ filename });
        return {
          content: [
            {
              type: "text",
              text: `Successfully deleted media file: ${filename}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to delete media file: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool("get_media_dir_path", {}, async () => {
    try {
      const path = await ankiClient.media.getMediaDirPath();
      return {
        content: [
          {
            type: "text",
            text: `Media directory path: ${path}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get media directory path: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool(
    "get_media_files_names",
    {
      pattern: z5.string().describe('Pattern to match media file names (e.g., "*.jpg", "*audio*")')
    },
    async ({ pattern }) => {
      try {
        const filenames = await ankiClient.media.getMediaFilesNames({ pattern });
        return {
          content: [
            {
              type: "text",
              text: `Media files matching pattern "${pattern}": ${JSON.stringify(filenames, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get media file names: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "retrieve_media_file",
    {
      filename: z5.string().describe("Name of the media file to retrieve")
    },
    async ({ filename }) => {
      try {
        const content = await ankiClient.media.retrieveMediaFile({ filename });
        if (content === false) {
          return {
            content: [
              {
                type: "text",
                text: `Media file "${filename}" not found`
              }
            ]
          };
        }
        return {
          content: [
            {
              type: "text",
              text: `Successfully retrieved media file "${filename}". Content length: ${content.length} characters`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to retrieve media file: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "store_media_file",
    {
      filename: z5.string().describe("Name to save the media file as"),
      data: z5.string().optional().describe("Base64-encoded data of the file"),
      path: z5.string().optional().describe("Local file path to read from"),
      url: z5.string().optional().describe("URL to download the file from"),
      deleteExisting: z5.boolean().optional().describe("Whether to delete existing file with same name")
    },
    async ({ filename, data, path, url, deleteExisting }) => {
      try {
        const params = { filename };
        if (data) params.data = data;
        if (path) params.path = path;
        if (url) params.url = url;
        if (deleteExisting !== void 0) params.deleteExisting = deleteExisting;
        if (!data && !path && !url) {
          throw new Error("Either data, path, or url must be provided");
        }
        const storedFilename = await ankiClient.media.storeMediaFile(params);
        return {
          content: [
            {
              type: "text",
              text: `Successfully stored media file as: ${storedFilename}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to store media file: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/tools/miscellaneous.ts
import { z as z6 } from "zod";
function registerMiscellaneousTools(server2) {
  server2.tool(
    "api_reflect",
    {
      actions: z6.array(z6.string()).nullable().optional().describe("List of action names to reflect, or null for all"),
      scopes: z6.array(z6.enum(["actions"])).describe("Scopes to reflect")
    },
    async ({ actions = null, scopes }) => {
      try {
        const reflection = await ankiClient.miscellaneous.apiReflect({ actions, scopes });
        return {
          content: [
            {
              type: "text",
              text: `API reflection: ${JSON.stringify(reflection, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get API reflection: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "export_package",
    {
      deckName: z6.string().describe("Name of the deck to export"),
      filePath: z6.string().describe("Path where to save the exported package"),
      includeSched: z6.boolean().optional().describe("Whether to include scheduling information")
    },
    async ({ deckName, filePath, includeSched }) => {
      try {
        const params = { deck: deckName, path: filePath };
        if (includeSched !== void 0) params.includeSched = includeSched;
        const result = await ankiClient.miscellaneous.exportPackage(params);
        return {
          content: [
            {
              type: "text",
              text: `Successfully exported deck "${deckName}" to ${filePath}. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to export package: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool("get_active_profile", {}, async () => {
    try {
      const profileName = await ankiClient.miscellaneous.getActiveProfile();
      return {
        content: [
          {
            type: "text",
            text: `Active profile: ${profileName}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get active profile: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("get_profiles", {}, async () => {
    try {
      const profiles = await ankiClient.miscellaneous.getProfiles();
      return {
        content: [
          {
            type: "text",
            text: `Available profiles: ${JSON.stringify(profiles, null, 2)}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get profiles: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool(
    "import_package",
    {
      filePath: z6.string().describe("Path to the package file to import")
    },
    async ({ filePath }) => {
      try {
        const result = await ankiClient.miscellaneous.importPackage({ path: filePath });
        return {
          content: [
            {
              type: "text",
              text: `Successfully imported package from ${filePath}. Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to import package: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "load_profile",
    {
      profileName: z6.string().describe("Name of the profile to load")
    },
    async ({ profileName }) => {
      try {
        const result = await ankiClient.miscellaneous.loadProfile({ name: profileName });
        return {
          content: [
            {
              type: "text",
              text: `Successfully loaded profile "${profileName}". Result: ${result}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to load profile: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "multi",
    {
      actions: z6.array(
        z6.object({
          action: z6.string().describe("Name of the action to execute"),
          params: z6.any().optional().describe("Parameters for the action"),
          version: z6.number().optional().describe("API version for the action")
        })
      ).describe("Array of actions to execute")
    },
    async ({ actions }) => {
      try {
        const results = await ankiClient.miscellaneous.multi({
          actions
        });
        return {
          content: [
            {
              type: "text",
              text: `Multi-action results: ${JSON.stringify(results, null, 2)}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to execute multi-action: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool("reload_collection", {}, async () => {
    try {
      await ankiClient.miscellaneous.reloadCollection();
      return {
        content: [
          {
            type: "text",
            text: "Successfully reloaded collection"
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to reload collection: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("request_permission", {}, async () => {
    try {
      const permission = await ankiClient.miscellaneous.requestPermission();
      return {
        content: [
          {
            type: "text",
            text: `Permission request result: ${JSON.stringify(permission, null, 2)}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to request permission: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("sync", {}, async () => {
    try {
      await ankiClient.miscellaneous.sync();
      return {
        content: [
          {
            type: "text",
            text: "Successfully initiated sync"
          }
        ]
      };
    } catch (error) {
      throw new Error(`Failed to sync: ${error instanceof Error ? error.message : String(error)}`);
    }
  });
  server2.tool("get_version", {}, async () => {
    try {
      const version = await ankiClient.miscellaneous.version();
      return {
        content: [
          {
            type: "text",
            text: `AnkiConnect version: ${version}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get version: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
}

// src/tools/model.ts
import { z as z7 } from "zod";
function registerModelTools(server2) {
  server2.tool(
    "create_model",
    {
      modelName: z7.string().describe("Name of the model to create"),
      inOrderFields: z7.array(z7.string()).describe("Array of field names in order"),
      cardTemplates: z7.array(
        z7.object({
          Front: z7.string().describe("Front template content"),
          Back: z7.string().describe("Back template content")
        }).and(z7.record(z7.string()))
      ).describe("Array of card templates with Front and Back content"),
      css: z7.string().optional().describe("Custom CSS styling for the model"),
      isCloze: z7.boolean().optional().describe("Whether the model should be a cloze type")
    },
    async ({ modelName, inOrderFields, cardTemplates, css, isCloze }) => {
      try {
        const result = await ankiClient.model.createModel({
          modelName,
          inOrderFields,
          cardTemplates,
          css,
          isCloze
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully created model "${modelName}" with ID: ${result.id}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to create model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "find_and_replace_in_models",
    {
      modelName: z7.string().describe("Name of the model to search in"),
      fieldText: z7.string().describe("Text to find"),
      replaceText: z7.string().describe("Text to replace with"),
      front: z7.boolean().describe("Whether to search in front templates"),
      back: z7.boolean().describe("Whether to search in back templates"),
      css: z7.boolean().describe("Whether to search in CSS")
    },
    async ({ modelName, fieldText, replaceText, front, back, css }) => {
      try {
        const count = await ankiClient.model.findAndReplaceInModels({
          model: {
            modelName,
            fieldText,
            replaceText,
            front,
            back,
            css
          }
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully replaced ${count} occurrences of "${fieldText}" with "${replaceText}" in model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to find and replace in model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_field_add",
    {
      modelName: z7.string().describe("Name of the model"),
      fieldName: z7.string().describe("Name of the field to add"),
      index: z7.number().describe("Position to insert the field (0-based)")
    },
    async ({ modelName, fieldName, index }) => {
      try {
        await ankiClient.model.modelFieldAdd({
          modelName,
          fieldName,
          index
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully added field "${fieldName}" to model "${modelName}" at position ${index}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to add field "${fieldName}" to model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_field_remove",
    {
      modelName: z7.string().describe("Name of the model"),
      fieldName: z7.string().describe("Name of the field to remove")
    },
    async ({ modelName, fieldName }) => {
      try {
        await ankiClient.model.modelFieldRemove({
          modelName,
          fieldName
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully removed field "${fieldName}" from model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to remove field "${fieldName}" from model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_field_rename",
    {
      modelName: z7.string().describe("Name of the model"),
      oldFieldName: z7.string().describe("Current name of the field"),
      newFieldName: z7.string().describe("New name for the field")
    },
    async ({ modelName, oldFieldName, newFieldName }) => {
      try {
        await ankiClient.model.modelFieldRename({
          modelName,
          oldFieldName,
          newFieldName
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully renamed field "${oldFieldName}" to "${newFieldName}" in model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to rename field in model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_field_reposition",
    {
      modelName: z7.string().describe("Name of the model"),
      fieldName: z7.string().describe("Name of the field to reposition"),
      index: z7.number().describe("New position for the field (0-based)")
    },
    async ({ modelName, fieldName, index }) => {
      try {
        await ankiClient.model.modelFieldReposition({
          modelName,
          fieldName,
          index
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully repositioned field "${fieldName}" to position ${index} in model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to reposition field "${fieldName}" in model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_field_set_description",
    {
      modelName: z7.string().describe("Name of the model"),
      fieldName: z7.string().describe("Name of the field"),
      index: z7.number().describe("Index of the field")
    },
    async ({ modelName, fieldName, index }) => {
      try {
        await ankiClient.model.modelFieldSetDescription({
          modelName,
          fieldName,
          index
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully set description for field "${fieldName}" in model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set description for field "${fieldName}" in model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_field_set_font",
    {
      modelName: z7.string().describe("Name of the model"),
      fieldName: z7.string().describe("Name of the field"),
      font: z7.string().describe("Font name to set")
    },
    async ({ modelName, fieldName, font }) => {
      try {
        await ankiClient.model.modelFieldSetFont({
          modelName,
          fieldName,
          font
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully set font "${font}" for field "${fieldName}" in model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set font for field "${fieldName}" in model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_field_set_font_size",
    {
      modelName: z7.string().describe("Name of the model"),
      fieldName: z7.string().describe("Name of the field"),
      fontSize: z7.number().describe("Font size to set")
    },
    async ({ modelName, fieldName, fontSize }) => {
      try {
        await ankiClient.model.modelFieldSetFontSize({
          modelName,
          fieldName,
          fontSize
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully set font size ${fontSize} for field "${fieldName}" in model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to set font size for field "${fieldName}" in model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_template_add",
    {
      modelName: z7.string().describe("Name of the model"),
      template: z7.object({
        Front: z7.string().describe("Front template content"),
        Back: z7.string().describe("Back template content")
      }).and(z7.record(z7.string())).describe("Template object with Front and Back content")
    },
    async ({ modelName, template }) => {
      try {
        await ankiClient.model.modelTemplateAdd({
          modelName,
          template
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully added template to model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to add template to model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_template_remove",
    {
      modelName: z7.string().describe("Name of the model"),
      templateName: z7.string().describe("Name of the template to remove")
    },
    async ({ modelName, templateName }) => {
      try {
        await ankiClient.model.modelTemplateRemove({
          modelName,
          templateName
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully removed template "${templateName}" from model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to remove template "${templateName}" from model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_template_rename",
    {
      modelName: z7.string().describe("Name of the model"),
      oldTemplateName: z7.string().describe("Current name of the template"),
      newTemplateName: z7.string().describe("New name for the template")
    },
    async ({ modelName, oldTemplateName, newTemplateName }) => {
      try {
        await ankiClient.model.modelTemplateRename({
          modelName,
          oldTemplateName,
          newTemplateName
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully renamed template "${oldTemplateName}" to "${newTemplateName}" in model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to rename template in model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "model_template_reposition",
    {
      modelName: z7.string().describe("Name of the model"),
      templateName: z7.string().describe("Name of the template to reposition"),
      index: z7.number().describe("New position for the template (0-based)")
    },
    async ({ modelName, templateName, index }) => {
      try {
        await ankiClient.model.modelTemplateReposition({
          modelName,
          templateName,
          index
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully repositioned template "${templateName}" to position ${index} in model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to reposition template "${templateName}" in model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "update_model_styling",
    {
      modelName: z7.string().describe("Name of the model"),
      css: z7.string().describe("New CSS styling for the model")
    },
    async ({ modelName, css }) => {
      try {
        await ankiClient.model.updateModelStyling({
          model: {
            name: modelName,
            css
          }
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully updated styling for model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to update styling for model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "update_model_templates",
    {
      modelName: z7.string().describe("Name of the model"),
      templates: z7.record(
        z7.object({
          Front: z7.string().optional().describe("Front template content"),
          Back: z7.string().optional().describe("Back template content")
        })
      ).describe("Object mapping template names to their Front/Back content")
    },
    async ({ modelName, templates }) => {
      try {
        await ankiClient.model.updateModelTemplates({
          model: {
            name: modelName,
            templates
          }
        });
        return {
          content: [
            {
              type: "text",
              text: `Successfully updated templates for model "${modelName}"`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to update templates for model "${modelName}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/tools/statistic.ts
import { z as z8 } from "zod";
function registerStatisticTools(server2) {
  server2.tool(
    "card_reviews",
    {
      deck: z8.string().describe("Name of the deck to get reviews for"),
      startID: z8.number().describe("Latest unix time not included in the result")
    },
    async ({ deck, startID }) => {
      try {
        const reviews = await ankiClient.statistic.cardReviews({ deck, startID });
        return {
          content: [
            {
              type: "text",
              text: `Found ${reviews.length} card reviews for deck "${deck}" after timestamp ${startID}`
            },
            {
              type: "text",
              text: JSON.stringify(reviews, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get card reviews for deck "${deck}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_collection_stats_html",
    {
      wholeCollection: z8.boolean().describe("Whether to get stats for the whole collection")
    },
    async ({ wholeCollection }) => {
      try {
        const statsHTML = await ankiClient.statistic.getCollectionStatsHTML({ wholeCollection });
        return {
          content: [
            {
              type: "text",
              text: `Collection statistics retrieved (${wholeCollection ? "whole collection" : "current deck"})`
            },
            {
              type: "text",
              text: statsHTML
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get collection statistics: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "get_latest_review_id",
    {
      deck: z8.string().describe("Name of the deck to get latest review ID for")
    },
    async ({ deck }) => {
      try {
        const reviewID = await ankiClient.statistic.getLatestReviewID({ deck });
        return {
          content: [
            {
              type: "text",
              text: `Latest review ID for deck "${deck}": ${reviewID}${reviewID === 0 ? " (no reviews found)" : ""}`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get latest review ID for deck "${deck}": ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool("get_num_cards_reviewed_by_day", {}, async () => {
    try {
      const reviewsByDay = await ankiClient.statistic.getNumCardsReviewedByDay();
      return {
        content: [
          {
            type: "text",
            text: `Cards reviewed by day (${reviewsByDay.length} days):`
          },
          {
            type: "text",
            text: JSON.stringify(reviewsByDay, null, 2)
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get cards reviewed by day: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool("get_num_cards_reviewed_today", {}, async () => {
    try {
      const reviewsToday = await ankiClient.statistic.getNumCardsReviewedToday();
      return {
        content: [
          {
            type: "text",
            text: `Number of cards reviewed today: ${reviewsToday}`
          }
        ]
      };
    } catch (error) {
      throw new Error(
        `Failed to get number of cards reviewed today: ${error instanceof Error ? error.message : String(error)}`
      );
    }
  });
  server2.tool(
    "get_reviews_of_cards",
    {
      cardIds: z8.array(z8.string()).describe("Array of card IDs to get reviews for")
    },
    async ({ cardIds }) => {
      try {
        const reviews = await ankiClient.statistic.getReviewsOfCards({ cards: cardIds });
        return {
          content: [
            {
              type: "text",
              text: `Reviews retrieved for ${cardIds.length} cards`
            },
            {
              type: "text",
              text: JSON.stringify(reviews, null, 2)
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to get reviews for cards: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
  server2.tool(
    "insert_reviews",
    {
      reviews: z8.array(
        z8.tuple([
          z8.number().describe("Review time (unix timestamp)"),
          z8.number().describe("Card ID"),
          z8.number().describe("USN"),
          z8.number().describe("Button pressed"),
          z8.number().describe("New interval"),
          z8.number().describe("Previous interval"),
          z8.number().describe("New factor"),
          z8.number().describe("Review duration"),
          z8.number().describe("Review type")
        ])
      ).describe("Array of review tuples to insert")
    },
    async ({ reviews }) => {
      try {
        await ankiClient.statistic.insertReviews({ reviews });
        return {
          content: [
            {
              type: "text",
              text: `Successfully inserted ${reviews.length} reviews into the database`
            }
          ]
        };
      } catch (error) {
        throw new Error(
          `Failed to insert reviews: ${error instanceof Error ? error.message : String(error)}`
        );
      }
    }
  );
}

// src/index.ts
var server = new McpServer6({
  name: "Anki MCP Server",
  version: "0.1.0"
});
registerDeckResources(server);
registerDeckTools(server);
registerCardResources(server);
registerCardTools(server);
registerNoteResources(server);
registerNoteTools(server);
registerModelResources(server);
registerModelTools(server);
registerStatisticResources(server);
registerStatisticTools(server);
registerGraphicalTools(server);
registerMediaTools(server);
registerMiscellaneousTools(server);
var transport = new StdioServerTransport();
await server.connect(transport);
//# sourceMappingURL=index.js.map